| package |
package := Package name: 'KeyboardLayout'.
package paxVersion: 0;
	basicComment: ''.


package classNames
	add: #KeyboardLayout;
	add: #LayoutTextPresenter;
	yourself.

package methodNames
	add: #UserLibrary -> #activateKeyboardLayout:uFlags:;
	add: #UserLibrary -> #getKeyboardLayout:;
	add: #UserLibrary -> #getKeyboardLayoutName:;
	yourself.

package globalNames
	add: #KeyboardLayoutConstants;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Documents and Settings\\My Documents\Dolphin Smalltalk 5.1\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\Documents and Settings\\My Documents\Dolphin Smalltalk 5.1\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	yourself).

package!

"Class Definitions"!

Object subclass: #KeyboardLayout
	instanceVariableNames: 'layouts'
	classVariableNames: 'Default'
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TextPresenter subclass: #LayoutTextPresenter
	instanceVariableNames: 'defaultLayout enterLayout'
	classVariableNames: ''
	poolDictionaries: 'KeyboardLayoutConstants'
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!UserLibrary methodsFor!

activateKeyboardLayout: hKl uFlags: uFlags
	"The ActivateKeyboardLayout function sets the input language for the current thread. This function accepts a keyboard layout handle that identifies a locale as well as the physical layout of the keyboard.

	HKL ActivateKeyboardLayout(

	    HKL hkl,	// handle to keyboard layout
	    UINT Flags 	// keyboard layout flags
	);"

	<stdcall: handle ActivateKeyboardLayout handle dword>
	^self invalidCall!

getKeyboardLayout: anInteger
	"The GetKeyboardLayout function retrieves the active keyboard layout for a specified thread. If the dwLayout parameter is zero, the layout for the active thread is returned.

	HKL GetKeyboardLayout(
	    DWORD dwLayout	// thread identifier
	);	
 	
	Parameters
		dwLayout	Identifies the thread to query or is zero for the current thread.

	Return Values
		The return value is the keyboard layout handle for the thread. The low word contains a locale identifier identifying the input language and the high word contains a device handle identifying the physical layout of the keyboard."

	<stdcall: handle GetKeyboardLayout dword>
	^self invalidCall!

getKeyboardLayoutName: lpszLayoutName
	"The GetKeyboardLayoutName function retrieves the name of the active keyboard layout. 

	BOOL GetKeyboardLayoutName(

	    LPTSTR pwszKLID 	// address of buffer for layout name
	);"

	<stdcall: bool GetKeyboardLayoutNameA lpstr>
	^self invalidCall! !
!UserLibrary categoriesFor: #activateKeyboardLayout:uFlags:!public!win32 functions-keyboard input! !
!UserLibrary categoriesFor: #getKeyboardLayout:!public!win32 functions-keyboard input! !
!UserLibrary categoriesFor: #getKeyboardLayoutName:!public!win32 functions-keyboard input! !

"End of package definition"!

"Source Globals"!

Smalltalk at: #KeyboardLayoutConstants put: (PoolConstantsDictionary named: #KeyboardLayoutConstants)!
KeyboardLayoutConstants at: 'ENGLISH_LAYOUT' put: 16r199!
KeyboardLayoutConstants at: 'RUSSIAN_LAYOUT' put: 16r1A3!
KeyboardLayoutConstants shrink!

"Classes"!

KeyboardLayout guid: (GUID fromString: '{E11675BC-3975-11D5-BA84-0008C789A424}')!
KeyboardLayout comment: '    .
    (#currentState)    (#activate:)'!
!KeyboardLayout categoriesForClass!Unclassified! !
!KeyboardLayout methodsFor!

activate: aNumber
	| layout |

	layout := self layouts at: aNumber ifAbsent: [ ^ self ].
	UserLibrary default activateKeyboardLayout: layout asExternalHandle uFlags: 0.
	!

currentState
	| layoutName |

	^ self layouts keyAtValue: (UserLibrary default getKeyboardLayout: 0) value ifAbsent: []!

initialize
	| bufferSize byteArray size currentLayoutHandle |

	bufferSize := 256.
	byteArray := ByteArray new: bufferSize.

	size := UserLibrary default getKeyboardLayoutList: bufferSize lpBuffer: byteArray.

	size isZero ifTrue: [ Error signal: ' 64  !!'].

	currentLayoutHandle := UserLibrary default getKeyboardLayout: 0.

	0 to: size -1 do: [ :i |
		| layout string |
		layout := byteArray dwordAtOffset: i*4.
		UserLibrary default activateKeyboardLayout: layout uFlags: 0.
		string := String new: 8.
		UserLibrary default getKeyboardLayoutName: string.
		self layouts at: (string rightString: 4) asNumber put: layout.
	].

	UserLibrary default activateKeyboardLayout: currentLayoutHandle uFlags: 0.
!

layouts
	layouts isNil ifTrue: [ layouts := LookupTable new ].
	^ layouts
	! !
!KeyboardLayout categoriesFor: #activate:!operations!public! !
!KeyboardLayout categoriesFor: #currentState!operations!public! !
!KeyboardLayout categoriesFor: #initialize!initializing!private! !
!KeyboardLayout categoriesFor: #layouts!accessing!initializing!private! !

!KeyboardLayout class methodsFor!

default

	^ Default ifNil: [ Default := super new initialize ]!

new
	self shouldNotImplement!

reset
	Default := nil! !
!KeyboardLayout class categoriesFor: #default!instance creation!public! !
!KeyboardLayout class categoriesFor: #new!instance creation!private! !
!KeyboardLayout class categoriesFor: #reset!initializing!public! !

LayoutTextPresenter guid: (GUID fromString: '{E11675CB-3975-11D5-BA84-0008C789A424}')!
LayoutTextPresenter comment: 'TextPresenter,       ,   .
.  .'!
!LayoutTextPresenter categoriesForClass!Unclassified! !
!LayoutTextPresenter methodsFor!

activatePreviousLayout
	enterLayout ifNil: [^self].
	[
		KeyboardLayout default activate: enterLayout.
	] on: Error do: [ :e | ].
	enterLayout := nil!

activateProperLayout

	self currentLayout ~= self defaultLayout ifTrue: [
		[ KeyboardLayout default activate: self defaultLayout ] on: Error do: [ :e | MessageBox errorMsg: '   !!',String lineDelimiter,'  .' ].
	]!

beEnglish
	defaultLayout := ENGLISH_LAYOUT!

beRussian
	defaultLayout := RUSSIAN_LAYOUT!

currentLayout
	^ [ KeyboardLayout default currentState ] on: Error do: [ :e | ]
!

defaultLayout
	^defaultLayout ifNil: [defaultLayout := ENGLISH_LAYOUT ]!

onKeyReleased: aKeyEvent

	super onKeyReleased: aKeyEvent.
	self activateProperLayout.!

onKillFocus
	super onKillFocus.
	self activatePreviousLayout!

onSetFocus
	
	super onSetFocus.

	enterLayout := self currentLayout.
	self activateProperLayout.! !
!LayoutTextPresenter categoriesFor: #activatePreviousLayout!operations!private! !
!LayoutTextPresenter categoriesFor: #activateProperLayout!operations!private! !
!LayoutTextPresenter categoriesFor: #beEnglish!accessing!private! !
!LayoutTextPresenter categoriesFor: #beRussian!accessing!private! !
!LayoutTextPresenter categoriesFor: #currentLayout!operations!private! !
!LayoutTextPresenter categoriesFor: #defaultLayout!accessing!private! !
!LayoutTextPresenter categoriesFor: #onKeyReleased:!event handling!public! !
!LayoutTextPresenter categoriesFor: #onKillFocus!event handling!public! !
!LayoutTextPresenter categoriesFor: #onSetFocus!event handling!public! !

!LayoutTextPresenter class methodsFor!

newEnglish
	^ self new beEnglish!

newRussian
	^ self new beRussian! !
!LayoutTextPresenter class categoriesFor: #newEnglish!instance creation!public! !
!LayoutTextPresenter class categoriesFor: #newRussian!instance creation!public! !

"Binary Globals"!

"Resources"!

